/********************************************************************************
 * Copyright (c) 2024 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/

import { Action, Args, hasStringProp, RequestAction } from '@eclipse-glsp/client';
import { TinaResponseAction } from './TinaResponseAction';

export interface RunTinaAction extends RequestAction<TinaResponseAction> {
    kind: typeof RunTinaAction.KIND;

    tinaArgs: string;

    /**
     * Custom arguments.
     */
    args?: Args;
}

export namespace RunTinaAction {
    export const KIND = 'RunTinaAction';

    export function is(object: unknown): object is RunTinaAction {
        return Action.hasKind(object, KIND) && hasStringProp(object, 'tinaArgs');
    }

    export function create(tinaArgs: string, options: { requestId?: string; args?: Args } = {}): RunTinaAction {
        return {
            kind: KIND,
            tinaArgs,
            requestId: '',
            ...options
        };
    }
}
