/********************************************************************************
 * Copyright (c) 2024 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/
import { SetUIExtensionVisibilityAction } from '@eclipse-glsp/client';
import { GLSPCommandHandler, GLSPContextMenu } from '@eclipse-glsp/theia-integration';
import { CommandContribution, CommandRegistry, MenuContribution, MenuModelRegistry } from '@theia/core';
import { ApplicationShell } from '@theia/core/lib/browser';
import { inject, injectable } from 'inversify';
import { TinaMenu } from 'tasklist-glsp';

export namespace TinaMenuCommand {
    export const TINA_MENU = 'tina-menu-command';
}

@injectable()
export class TinaMenuCommandContribution implements CommandContribution {
    @inject(ApplicationShell) protected readonly shell: ApplicationShell;
    registerCommands(commands: CommandRegistry): void {
        commands.registerCommand(
            { id: TinaMenuCommand.TINA_MENU, label: 'Tina' },
            new GLSPCommandHandler(this.shell, {
                actions: context => [
                    SetUIExtensionVisibilityAction.create({
                        extensionId: TinaMenu.ID,
                        visible: true
                    })
                ],
                isEnabled: context => !context.isReadonly
            })
        );
    }
}

@injectable()
export class TinaMenuMenuContribution implements MenuContribution {
    static readonly EDIT = GLSPContextMenu.MENU_PATH.concat('edit');
    registerMenus(menus: MenuModelRegistry): void {
        menus.registerMenuAction(TinaMenuMenuContribution.EDIT, { commandId: TinaMenuCommand.TINA_MENU });
    }
}
