/********************************************************************************
 * Copyright (c) 2022 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied:
 * -- GNU General Public License, version 2 with the GNU Classpath Exception
 * which is available at https://www.gnu.org/software/classpath/license.html
 * -- MIT License which is available at https://opensource.org/license/mit.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR MIT
 ********************************************************************************/
import { ContainerContext, DiagramConfiguration, GLSPTheiaFrontendModule } from '@eclipse-glsp/theia-integration';
import { CommandContribution, MenuContribution } from '@theia/core';
import { TaskListLanguage } from '../common/tasklist-language';
import { CopyIdCommandContribution, CopyIdMenuContribution } from './diagram/copy-id-context-menu';
import { EditInputSignalsMenuCommandContribution, EditInputSignalsMenuMenuContribution } from './diagram/edit-input-signals-context-menu';
import { EditTokensMenuCommandContribution, EditTokensMenuMenuContribution } from './diagram/edit-tokens-context-menu';
import { FireTransitionCommandContribution, FireTransitionMenuContribution } from './diagram/fire-transtition-context-menu';
import { ResetMarkingsCommandContribution, ResetMarkingsMenuContribution } from './diagram/reset-markings-context-menu';
import { SignalClauseMenuCommandContribution, SignalClauseMenuMenuContribution } from './diagram/signal-clause-context-menu';
import { TaskListDiagramConfiguration } from './diagram/tasklist-diagram-configuration';
import { TinaMenuCommandContribution, TinaMenuMenuContribution } from './diagram/tina-context-menu';

export class TaskListTheiaFrontendModule extends GLSPTheiaFrontendModule {
    readonly diagramLanguage = TaskListLanguage;

    bindDiagramConfiguration(context: ContainerContext): void {
        context.bind(DiagramConfiguration).to(TaskListDiagramConfiguration);
    }

    override configure(context: ContainerContext): void {
        context.bind(CommandContribution).to(FireTransitionCommandContribution);
        context.bind(MenuContribution).to(FireTransitionMenuContribution);
        context.bind(CommandContribution).to(ResetMarkingsCommandContribution);
        context.bind(MenuContribution).to(ResetMarkingsMenuContribution);
        context.bind(CommandContribution).to(TinaMenuCommandContribution);
        context.bind(MenuContribution).to(TinaMenuMenuContribution);
        context.bind(CommandContribution).to(SignalClauseMenuCommandContribution);
        context.bind(MenuContribution).to(SignalClauseMenuMenuContribution);
        context.bind(CommandContribution).to(EditInputSignalsMenuCommandContribution);
        context.bind(MenuContribution).to(EditInputSignalsMenuMenuContribution);
        context.bind(CommandContribution).to(EditTokensMenuCommandContribution);
        context.bind(MenuContribution).to(EditTokensMenuMenuContribution);
        context.bind(CommandContribution).to(CopyIdCommandContribution);
        context.bind(MenuContribution).to(CopyIdMenuContribution);
    }
}

export default new TaskListTheiaFrontendModule();
