package org.eclipse.glsp.example.javaemf.MQTT;


import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.inject.Inject;
import org.eclipse.glsp.example.javaemf.Simulation.FireTransitionExecutor;
import org.eclipse.glsp.example.javaemf.Simulation.FireTransitionOperation;
import org.eclipse.glsp.example.javaemf.model.ExtendedModelState;
import org.eclipse.glsp.example.javaemf.operation.SetDiNeROSTokenOperation;
import org.eclipse.glsp.example.javaemf.operation.SetInputSignalOperation;
import org.eclipse.glsp.example.javaemf.operation.SetPageVisibilityOperation;
import org.eclipse.glsp.server.actions.ActionDispatcher;
import org.eclipse.glsp.server.features.core.model.UpdateModelAction;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.pnml.emf.ptnet.Transition;

import java.util.Arrays;
import java.util.HashSet;

public class MQTTClient {

    @Inject
    protected ExtendedModelState modelState;

    @Inject
    protected ActionDispatcher actionDispatcher;

    @Inject
    protected FireTransitionExecutor fireTransitionExecutor;

    String broker = "tcp://broker.emqx.io:1883";
    String fireTransitionTopic = "fireTransition";

    String fillPlaceTopic = "fillPlace";

    HashSet<String> signalTopics = new HashSet<>();

    String clientId = "pnml_client";
    int qos = 0;

    MqttClient mqttClient;

    Gson gson = new Gson();

    public MQTTClient () {
        try {


            mqttClient = new MqttClient(broker, clientId, new MemoryPersistence());
            MqttConnectOptions options = new MqttConnectOptions();
            options.setConnectionTimeout(60);
            options.setKeepAliveInterval(60);

            mqttClient.setCallback(new MqttCallback() {

                public void connectionLost(Throwable cause) {
                    System.out.println("connectionLost: " + cause.getMessage());
                }

                public void messageArrived(String topic, MqttMessage message) {
                    String messageString = new String(message.getPayload());

                    System.out.println("topic: " + topic);
                    System.out.println("message: " + messageString);

                    if(fireTransitionTopic.equals(topic)){
                        fireTransition(messageString);
                        return;
                    }

                    if(fillPlaceTopic.equals(topic)){
                        fillPlace(messageString);
                        return;
                    }

                    if (signalTopics.contains(topic) && modelState != null){
                        setSignal(topic ,messageString);
                        return;
                    }

                }

                public void deliveryComplete(IMqttDeliveryToken token) {
                    System.out.println("Delivery complete.");
                }

            });
            mqttClient.connect(options);
            mqttClient.subscribe(fireTransitionTopic, qos);
            mqttClient.subscribe(fillPlaceTopic, qos);

            System.out.println("mqttclient registered");
        } catch (MqttException e){
            e.printStackTrace();
        }
    }

    protected void setSignal(String topic, String message){
        // will set signal to false if message is something other than a Boolean value
        boolean signalValue = Boolean.parseBoolean(message);
        actionDispatcher.dispatch(new SetInputSignalOperation(topic, signalValue));
    }


    public void subscribe(String topic){
        if(signalTopics.contains(topic)){
            return;
        }
        try{
            mqttClient.subscribe(topic, qos);
            signalTopics.add(topic);
            System.out.println("Subscribe: " + topic);

        } catch (MqttException e){
            e.printStackTrace();
        }
    }

    public void unsubscribe(String topic){
        if(!signalTopics.contains(topic)){
            return;
        }
        try{
            mqttClient.unsubscribe(topic);
            signalTopics.remove(topic);
            System.out.println("Unsubscribe: " + topic);
        } catch (MqttException e){
            e.printStackTrace();
        }

    }

    public HashSet<String> getSignalTopics() {
        return signalTopics;
    }

    public void setSignalTopics(HashSet<String> signalTopics) {
        this.signalTopics = signalTopics;
    }

    protected void fireTransition(String message){
        try {
            FireTransitionMQTTMessage fireTransitionMessage = gson.fromJson(message, FireTransitionMQTTMessage.class);
            if(fireTransitionMessage.getTokens() != null){
                actionDispatcher.dispatch(new FireTransitionOperation(fireTransitionMessage.getTransitionId(), fireTransitionMessage.getTokens()));
            } else {
                actionDispatcher.dispatch(new FireTransitionOperation(fireTransitionMessage.getTransitionId()));

            }
        } catch (JsonSyntaxException e){
            e.printStackTrace();
        }
    }

    protected void fillPlace(String message){
        try {
            FillPlaceMQTTMessage fillPlaceMessage = gson.fromJson(message, FillPlaceMQTTMessage.class);
            actionDispatcher.dispatch(new SetDiNeROSTokenOperation(fillPlaceMessage.getPlaceId(), Arrays.asList(fillPlaceMessage.getTokens())));
        } catch (JsonSyntaxException e){
            e.printStackTrace();
        }
    }
}
