package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ExtendedModelState;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.server.emf.EMFOperationHandler;
import org.eclipse.glsp.server.operations.Operation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;

public abstract class AbstractDiNeROSTokenHandler<T extends Operation> extends EMFOperationHandler<T> {
    @Inject
    protected ExtendedModelState modelState;

    @Inject
    protected ModelAccessUtil modelAccessUtil;
    protected ToolInfoPlace getOrCreateToolInfo(Place place, CompoundCommand command){
        Optional<ToolInfoPlace> optToolInfoPlace = modelAccessUtil.getDiNeROSPlaceToolSpecific(place);
        if(optToolInfoPlace.isEmpty()) {
            ToolInfoPlace newToolInfo = PtnetFactory.eINSTANCE.createToolInfoPlace();
            newToolInfo.setTool(PNMLModelTypes.DINEROS_TOOL_NAME);
            newToolInfo.setVersion(PNMLModelTypes.DINEROS_VERSION);

            command.append(AddCommand.create(modelState.getEditingDomain(), place,
                PtnetPackage.Literals.PLACE__TOOLSPECIFIC, newToolInfo));

            return newToolInfo;
        }
        return optToolInfoPlace.get();
    }

    protected BalloonMarking getOrCreateBalloonMarking(ToolInfoPlace toolInfoPlace, CompoundCommand command) {
        BalloonMarking balloonMarking = toolInfoPlace.getBalloonMarking();
        if (balloonMarking == null) {
            balloonMarking = PtnetFactory.eINSTANCE.createBalloonMarking();
            command.append(SetCommand.create(modelState.getEditingDomain(), toolInfoPlace,
                PtnetPackage.Literals.TOOL_INFO_PLACE__BALLOON_MARKING, balloonMarking));
        }
        return balloonMarking;
    }

    protected Tokens getOrCreateTokens(BalloonMarking balloonMarking, CompoundCommand command) {
        Tokens tokens = balloonMarking.getTokens();
        if (tokens == null) {
            tokens = PtnetFactory.eINSTANCE.createTokens();
            command.append(SetCommand.create(modelState.getEditingDomain(), balloonMarking,
                PtnetPackage.Literals.BALLOON_MARKING__TOKENS, tokens));
        }
        return tokens;
    }

    protected void setInitialMarking(Place place, int marking, CompoundCommand command){
        PTMarking newMarking = PtnetFactory.eINSTANCE.createPTMarking();
        newMarking.setText(marking);
        command.append(SetCommand.create(modelState.getEditingDomain(), place,
            PtnetPackage.Literals.PLACE__INITIAL_MARKING, newMarking));
    }
}
