package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ExtendedModelState;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.example.javaemf.operation.AddInputSignalOperation;
import org.eclipse.glsp.server.emf.EMFOperationHandler;
import org.pnml.emf.ptnet.*;

import java.util.Objects;
import java.util.Optional;

public class AddInputSignalHandler extends EMFOperationHandler<AddInputSignalOperation> {
    @Inject
    protected ExtendedModelState modelState;

    @Inject
    protected ModelAccessUtil modelAccessUtil;

    @Override
    public Optional<Command> createCommand(AddInputSignalOperation operation) {
        pnml netContainer = modelState.getSemanticModel(pnml.class).orElseThrow();
        PetriNet petriNet = netContainer.getNet().stream().findFirst().orElseThrow();

        String inputSignalId = operation.getInputSignalId();

        if(inputSignalId == null || inputSignalId.isEmpty()){
            throw new IllegalArgumentException("Input signal id must be an non empty string");
        }

        InputSignal inputSignal = PtnetFactory.eINSTANCE.createInputSignal();
        inputSignal.setInputsignalID(operation.getInputSignalId());
        inputSignal.setInitialvalue(false);

        Optional<ToolInfoNet> optNetInfo = modelAccessUtil.getDiNeROSNetToolSpecific(petriNet);
        if (optNetInfo.isEmpty()){
            ToolInfoNet newToolInfo = PtnetFactory.eINSTANCE.createToolInfoNet();
            newToolInfo.setTool(PNMLModelTypes.DINEROS_TOOL_NAME);
            newToolInfo.setVersion(PNMLModelTypes.DINEROS_VERSION);

            InputSignals newInputSignals = PtnetFactory.eINSTANCE.createInputSignals();
            newInputSignals.getInputsignal().add(inputSignal);
            newToolInfo.setInputsignals(newInputSignals);

            return Optional.of(AddCommand.create(modelState.getEditingDomain(), petriNet,
                PtnetPackage.Literals.PETRI_NET__TOOLSPECIFIC, newToolInfo));
        }

        ToolInfoNet netInfo = optNetInfo.get();
        if (netInfo.getInputsignals() == null){
            InputSignals newInputSignals = PtnetFactory.eINSTANCE.createInputSignals();
            newInputSignals.getInputsignal().add(inputSignal);

            return Optional.of(SetCommand.create(modelState.getEditingDomain(), netInfo,
                PtnetPackage.Literals.TOOL_INFO_NET__INPUTSIGNALS, newInputSignals));
        }

        if(netInfo.getInputsignals().getInputsignal().stream()
            .anyMatch(signal -> Objects.equals(signal.getInputsignalID(), inputSignalId))){
            // Signal id already exists
            return Optional.of(new CompoundCommand());
        }

        return Optional.of(AddCommand.create(modelState.getEditingDomain(), netInfo.getInputsignals(),
            PtnetPackage.Literals.INPUT_SIGNALS__INPUTSIGNAL, inputSignal));

    }
}