package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.glsp.example.javaemf.operation.DeleteDiNeROSTokenOperation;
import org.pnml.emf.ptnet.*;

import java.util.Objects;
import java.util.Optional;

public class DeleteDiNeROSTokenHandler extends AbstractDiNeROSTokenHandler<DeleteDiNeROSTokenOperation> {

    @Override
    public Optional<Command> createCommand(DeleteDiNeROSTokenOperation operation) {
        Place place = modelState.getIndex().getEObject(operation.getPlaceId(), Place.class).orElseThrow(() ->
            new IllegalArgumentException("No place with this id exists"));

        CompoundCommand command = new CompoundCommand();
        String token = operation.getToken();

        Optional<Tokens> optTokens = modelAccessUtil.getDiNeROSPlaceToolSpecific(place)
            .map(ToolInfoPlace::getBalloonMarking)
            .map(BalloonMarking::getTokens);
        if(optTokens.isEmpty()){
            throw new IllegalArgumentException("Place has no tokens");
        }

        Tokens tokens = optTokens.get();

        boolean tokenInList = tokens.getToken().stream().anyMatch(s -> Objects.equals(s, token));

        if(!tokenInList){
            throw new IllegalArgumentException("Place does not contain this token");
        }

        setInitialMarking(place, tokens.getToken().size()-1, command);

        command.append(RemoveCommand.create(modelState.getEditingDomain(), tokens,
            PtnetPackage.Literals.TOKENS__TOKEN, token));

        return Optional.of(command);
    }

}






















