package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.glsp.example.javaemf.model.ExtendedModelState;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.example.javaemf.operation.DeleteInputSignalOperation;
import org.eclipse.glsp.server.emf.EMFOperationHandler;
import org.pnml.emf.ptnet.*;

import java.util.Objects;
import java.util.Optional;

public class DeleteInputSignalHandler extends EMFOperationHandler<DeleteInputSignalOperation> {
    @Inject
    protected ExtendedModelState modelState;

    @Inject
    protected ModelAccessUtil modelAccessUtil;


    @Override
    public Optional<Command> createCommand(DeleteInputSignalOperation operation) {
        pnml netContainer = modelState.getSemanticModel(pnml.class).orElseThrow();
        PetriNet petriNet = netContainer.getNet().stream().findFirst().orElseThrow();
        ToolInfoNet netInfo = modelAccessUtil.getDiNeROSNetToolSpecific(petriNet).orElseThrow();
        InputSignals inputSignals = netInfo.getInputsignals();
        if(inputSignals == null) {
            throw new IllegalArgumentException("Net has no input signals");
        }

        String signalId = operation.getInputSignalId();

        InputSignal inputSignal = inputSignals.getInputsignal().stream().filter(signal ->
            Objects.equals(signal.getInputsignalID(), signalId)
        ).findFirst().orElseThrow(() -> new IllegalArgumentException("Signal does not exist"));

        return Optional.of(RemoveCommand.create(modelState.getEditingDomain(), inputSignals,
            PtnetPackage.Literals.INPUT_SIGNALS__INPUTSIGNAL, inputSignal));

    }
}