package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ExtendedModelState;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.example.javaemf.operation.EditSignalClauseOperation;
import org.eclipse.glsp.example.javaemf.validators.InputSignalValidator;
import org.eclipse.glsp.server.emf.EMFOperationHandler;
import org.pnml.emf.ptnet.PtnetFactory;
import org.pnml.emf.ptnet.PtnetPackage;
import org.pnml.emf.ptnet.ToolInfoTransition;
import org.pnml.emf.ptnet.Transition;

import java.util.Optional;

public class EditSignalClauseHandler extends EMFOperationHandler<EditSignalClauseOperation> {
    @Inject
    protected ExtendedModelState modelState;

    @Inject
    protected ModelAccessUtil modelAccessUtil;

    @Inject
    protected InputSignalValidator signalValidator;

    @Override
    public Optional<Command> createCommand(EditSignalClauseOperation operation) {
        Transition transition = (Transition) modelState.getIndex().getEObject(operation.getTransitionId()).orElseThrow();
        String clause = operation.getSignalClause();

        if (!signalValidator.isValidCNF(clause)){
            throw new IllegalArgumentException("Input must be in CNF");
        }

        if (!modelAccessUtil.isDiscreteTransition(transition)){
            throw new IllegalArgumentException("Only regular/discrete transitions can have signal clauses");
        }

        Optional<ToolInfoTransition> optToolSpecifics = modelAccessUtil.getDiNeROSTransitionToolSpecific(transition, PNMLModelTypes.DISCRETE_TRANSITION);
        ToolInfoTransition toolSpecifics;
        if (optToolSpecifics.isPresent()){
            toolSpecifics = optToolSpecifics.get();
            return Optional.of(SetCommand.create(modelState.getEditingDomain(), toolSpecifics,
                PtnetPackage.Literals.TOOL_INFO_TRANSITION__INPUTSIGNALCLAUSE, clause));

        } else {
            toolSpecifics = PtnetFactory.eINSTANCE.createToolInfoTransition();
            toolSpecifics.setTool(PNMLModelTypes.DINEROS_TOOL_NAME);
            toolSpecifics.setVersion(PNMLModelTypes.DINEROS_VERSION);
            toolSpecifics.setType(PNMLModelTypes.DISCRETE_TRANSITION);
            toolSpecifics.setInputsignalclause(clause);
            return Optional.of(AddCommand.create(modelState.getEditingDomain(), transition,
                PtnetPackage.Literals.TRANSITION__TOOLSPECIFIC, toolSpecifics));
        }
    }
}
