package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;

import jakarta.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.graph.GraphPackage;
import org.eclipse.glsp.server.emf.EMFCreateOperationHandler;
import org.eclipse.glsp.server.operations.CreateNodeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;

public class CreateServiceChannelHandler extends EMFCreateOperationHandler<CreateNodeOperation> {
    @Inject
    protected ModelAccessUtil modelAccessUtil;

    public CreateServiceChannelHandler() {
        super(PNMLModelTypes.SERVICE_CHANNEL);
    }

    @Override
    public String getLabel() {
        return "ServiceChannel";
    }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        EditingDomain editingDomain = modelState.getEditingDomain();
        Transition transition = (Transition) modelState.getIndex().getEObject(operation.getContainerId()).orElseThrow();
        if (!modelAccessUtil.isServiceTransition(transition)) {
            throw new IllegalArgumentException("Channels can only be created on Service nodes");
        }

        CompoundCommand command = new CompoundCommand();

        ToolInfoTransition toolSpecifics = transition.getToolspecific().stream().filter(toolInfo ->
                Objects.equals(toolInfo.getType(), PNMLModelTypes.SERVICE_TRANSITION))
            .findFirst().orElseThrow();

        Channels channels = toolSpecifics.getChannels();
        if (channels == null) {
            channels = PtnetFactory.eINSTANCE.createChannels();
            command.append(SetCommand.create(editingDomain, toolSpecifics,
                PtnetPackage.Literals.TOOL_INFO_TRANSITION__CHANNELS, channels));
        }

        Channel channel = PtnetFactory.eINSTANCE.createChannel();

        Function<Integer, String> nameProvider = i -> "Channel" + i;
        int transitionCounter = modelState.getIndex().getCounter(GraphPackage.Literals.GNODE, nameProvider);
        channel.setCid(nameProvider.apply(transitionCounter)); //TODO think of something better


        command.append(AddCommand.create(editingDomain, channels,
            PtnetPackage.Literals.CHANNELS__CHANNEL, channel));

        return Optional.of(command);
    }
}