package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.handler.NodeCreation.AbstractCreateNodeHandler;
import org.eclipse.glsp.graph.GPoint;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;

public class CreateServiceTransitionHandler extends AbstractCreateNodeHandler<CreateNodeOperation, Transition> {
    public CreateServiceTransitionHandler() {
        super(PNMLModelTypes.SERVICE_TRANSITION);
    }

    @Override
    public String getLabel() { return "ServiceTransition"; }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        EditingDomain editingDomain = modelState.getEditingDomain();
        Optional<GPoint> relativeLocation = getRelativeLocation(operation);
        Page page = getContainerPage(operation);

        Transition newTransition = PtnetFactory.eINSTANCE.createTransition();
        setElementId(newTransition);
        setInitialName(newTransition, "Service");

        NodeGraphics shape = createShape(idGenerator.getOrCreateId(newTransition),
            relativeLocation, GraphUtil.dimension(PNMLModelTypes.DefaultSizes.TopicWidth, PNMLModelTypes.DefaultSizes.TopicHeight));
        newTransition.setNodegraphics(shape);

        ToolInfoTransition toolSpecifics = PtnetFactory.eINSTANCE.createToolInfoTransition();
        toolSpecifics.setTool(PNMLModelTypes.DINEROS_TOOL_NAME);
        toolSpecifics.setVersion(PNMLModelTypes.DINEROS_VERSION);
        toolSpecifics.setType(PNMLModelTypes.SERVICE_TRANSITION);
        toolSpecifics.setServiceName(newTransition.getName().getText());
        toolSpecifics.setServerCapacity(0);
        newTransition.getToolspecific().add(toolSpecifics);


        Command transitionCommand = AddCommand.create(editingDomain, page,
            PtnetPackage.Literals.PAGE__TRANSITION, newTransition);

        return Optional.of(transitionCommand);
    }
}
