package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.server.emf.EMFCreateOperationHandler;
import org.eclipse.glsp.server.operations.CreateEdgeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Objects;
import java.util.Optional;



public class CreateTopicLinkHandler extends EMFCreateOperationHandler<CreateEdgeOperation> {
    @Inject
    protected ModelAccessUtil modelAccessUtil;

    public CreateTopicLinkHandler() {
        super(PNMLModelTypes.TOPIC_LINK);
    }


    @Override
    public String getLabel() { return "TopicLink"; }

    @Override
    public Optional<Command> createCommand(CreateEdgeOperation createEdgeOperation) {
        EditingDomain editingDomain = modelState.getEditingDomain();

        String sourceId = createEdgeOperation.getSourceElementId();
        String targetId = createEdgeOperation.getTargetElementId();

        Node source = (Node) modelState.getIndex().getEObject(sourceId).orElseThrow();
        Node target = (Node) modelState.getIndex().getEObject(targetId).orElseThrow();

        CompoundCommand command = new CompoundCommand();


        if(PtnetPackage.eINSTANCE.getTransition().isSuperTypeOf(target.eClass())
            && modelAccessUtil.isTopicTransition((Transition) target)
            && PtnetPackage.eINSTANCE.getPlaceNode().isSuperTypeOf(source.eClass())
        ) {
            // edge was drawn from place to topic
            // therefore publisher link is created
            Publisher publisher = PtnetFactory.eINSTANCE.createPublisher();
            publisher.setId(sourceId);
            publisher.setLimit(PNMLModelTypes.TOPIC_DEFAULT_LIMIT);

            ToolInfoTransition toolSpecifics = ((Transition) target).getToolspecific().stream().filter(toolInfo -> Objects.equals(toolInfo.getType(), PNMLModelTypes.TOPIC_TRANSITION))
                .findFirst().orElseThrow();

            Publishers publishers = toolSpecifics.getPublishers();
            if(publishers == null){
                publishers = PtnetFactory.eINSTANCE.createPublishers();
                command.append(SetCommand.create(editingDomain, toolSpecifics,
                    PtnetPackage.Literals.TOOL_INFO_TRANSITION__PUBLISHERS, publishers));
            }

            command.append(AddCommand.create(editingDomain, publishers,
                PtnetPackage.Literals.PUBLISHERS__PUBLISHER, publisher));

        } else if (PtnetPackage.eINSTANCE.getTransition().isSuperTypeOf(source.eClass())
            && modelAccessUtil.isTopicTransition((Transition) source)
            && PtnetPackage.eINSTANCE.getPlaceNode().isSuperTypeOf(target.eClass())
        ){
            // edge was drawn from topic to place
            // therefore subscriber link is created
            Subscriber subscriber = PtnetFactory.eINSTANCE.createSubscriber();
            subscriber.setId(targetId);
            subscriber.setLimit(PNMLModelTypes.TOPIC_DEFAULT_LIMIT);

            ToolInfoTransition toolSpecifics = ((Transition) source).getToolspecific().stream().filter(toolInfo -> Objects.equals(toolInfo.getType(), PNMLModelTypes.TOPIC_TRANSITION))
                .findFirst().orElseThrow();

            Subscribers subscribers = toolSpecifics.getSubscribers();
            if(subscribers == null){
                subscribers = PtnetFactory.eINSTANCE.createSubscribers();
                command.append(SetCommand.create(editingDomain, toolSpecifics,
                    PtnetPackage.Literals.TOOL_INFO_TRANSITION__SUBSCRIBERS, subscribers));
            }

            command.append(AddCommand.create(editingDomain, subscribers,
                PtnetPackage.Literals.SUBSCRIBERS__SUBSCRIBER, subscriber));

        } else {
            throw new IllegalArgumentException("Topic link must be between topic and place node.");
        }


        return Optional.of(command);
    }
}
