package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.glsp.example.javaemf.operation.SetDiNeROSTokenOperation;
import org.pnml.emf.ptnet.*;

import java.util.List;
import java.util.Optional;

public class SetDiNeROSTokenHandler extends AbstractDiNeROSTokenHandler<SetDiNeROSTokenOperation> {

    @Override
    public Optional<Command> createCommand(SetDiNeROSTokenOperation operation) {
        Place place = modelState.getIndex().getEObject(operation.getPlaceId(), Place.class).orElseThrow(() ->
            new IllegalArgumentException("No place with this id exists"));

        //Uses a compound Command to potentially append the creation commands for toolInfo, balloonMarkings (if necessary)
        CompoundCommand command = new CompoundCommand();
        List<String> balloonTokens = operation.getToken();

        ToolInfoPlace toolInfoPlace = getOrCreateToolInfo(place, command);
        BalloonMarking balloonMarking = getOrCreateBalloonMarking(toolInfoPlace, command);
        Tokens tokens = PtnetFactory.eINSTANCE.createTokens();
        tokens.getToken().addAll(balloonTokens);

        setInitialMarking(place, balloonTokens.size(), command);

        command.append(SetCommand.create(modelState.getEditingDomain(), balloonMarking,
            PtnetPackage.Literals.BALLOON_MARKING__TOKENS, tokens));
        return Optional.of(command);
    }

}






















