package org.eclipse.glsp.example.javaemf.handler.EdgeAndRefCreation;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GraphPackage;
import org.eclipse.glsp.server.emf.EMFCreateOperationHandler;
import org.eclipse.glsp.server.emf.EMFIdGenerator;
import org.eclipse.glsp.server.operations.CreateEdgeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;
import java.util.UUID;
import java.util.function.Function;

public class CreateArcHandler extends EMFCreateOperationHandler<CreateEdgeOperation> {

    @Inject
    protected EMFIdGenerator idGenerator;

    public CreateArcHandler() {
        super(PNMLModelTypes.ARC);
    }

    public CreateArcHandler(String type) {
        super(type);
    }


    @Override
    public String getLabel() { return "Arc"; }

    @Override
    public Optional<Command> createCommand(CreateEdgeOperation createEdgeOperation) {
        Arc newArc = PtnetFactory.eINSTANCE.createArc();
        newArc.setId(UUID.randomUUID().toString());
        setInitialName(newArc);

        setType(newArc);

        String sourceId = createEdgeOperation.getSourceElementId();
        String targetId = createEdgeOperation.getTargetElementId();

        Node source = modelState.getIndex().getEObject(sourceId, Node.class).orElseThrow();
        Node target = modelState.getIndex().getEObject(targetId, Node.class).orElseThrow();

        Page page = (Page) (source.eContainer());

        return Optional.of(createArc(newArc, source, target, page));
    }

    protected void setInitialName(final Arc arc) {
        Function<Integer, String> nameProvider = i -> "New" + arc.eClass().getName() + i;
        int ArcCounter = modelState.getIndex().getCounter(GraphPackage.Literals.GEDGE, nameProvider);
        Name newName = PtnetFactory.eINSTANCE.createName();
        newName.setText(nameProvider.apply(ArcCounter));
        arc.setName(newName);
    }

    protected Command createArc(Arc newArc, Node source, Node target, Page page) {
        EditingDomain editingDomain = modelState.getEditingDomain();

        //explicitly set the arcs to the nodes (source / target) so the operation is added to the command stack
        // and can be undone. Otherwise, the reference would not be removed from the nodes.

        Command arcCommand = AddCommand.create(editingDomain, page,
            PtnetPackage.Literals.PAGE__ARC, newArc);

        Command setSource = SetCommand.create(editingDomain, newArc,
            PtnetPackage.Literals.ARC__SOURCE, source);

        Command setTarget = SetCommand.create(editingDomain, newArc,
            PtnetPackage.Literals.ARC__TARGET, target);

        CompoundCommand compoundCommand = new CompoundCommand();

        compoundCommand.append(arcCommand);
        compoundCommand.append(setSource);
        compoundCommand.append(setTarget);

        return compoundCommand;
    }

    protected void setType(Arc arc){
        //empty because normal arcs do not have a type attribute in DiNeROS
        //used as template method for subclasses (inhibitor, read)
    }

}
