package org.eclipse.glsp.example.javaemf.handler.NodeCreation;

import java.util.Optional;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GPoint;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;

import org.pnml.emf.ptnet.*;


public class CreatePlaceHandler extends AbstractCreateNodeHandler<CreateNodeOperation, Place> {

    public CreatePlaceHandler() {
        super(PNMLModelTypes.PLACE);
    }

    @Override
    public String getLabel() {
        return "Place";
    }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        EditingDomain editingDomain = modelState.getEditingDomain();
        Optional<GPoint> relativeLocation = getRelativeLocation(operation);
        Page page = getContainerPage(operation);

        Place newPlace = PtnetFactory.eINSTANCE.createPlace();
        initializeElement(newPlace);

        NodeGraphics shape = createShape(idGenerator.getOrCreateId(newPlace),
            relativeLocation, GraphUtil.dimension(PNMLModelTypes.DefaultSizes.PlaceWidth, PNMLModelTypes.DefaultSizes.PlaceHeight));
        newPlace.setNodegraphics(shape);

        setDiNeROSPlaceInfo(newPlace, page.getId());

        Command placeCommand = AddCommand.create(editingDomain, page,
            PtnetPackage.Literals.PAGE__PLACE, newPlace);

        return Optional.of(placeCommand);
    }

    protected void setDiNeROSPlaceInfo(Place place, String pageNodeId){
        ToolInfoPlace toolInfo = PtnetFactory.eINSTANCE.createToolInfoPlace();
        setDiNeROSToolSpecifics(toolInfo, pageNodeId);
        place.getToolspecific().add(toolInfo);
    }
}

