package org.eclipse.glsp.example.javaemf.handler.NodeCreation;

import java.util.Optional;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GPoint;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;

import org.pnml.emf.ptnet.*;


public class CreateReferenceTransitionHandler extends AbstractCreateNodeHandler<CreateNodeOperation, RefTransition> {

    public CreateReferenceTransitionHandler() {
        super(PNMLModelTypes.REF_TRANSITION);
    }

    @Override
    public String getLabel() {
        return "Reference Transition";
    }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        EditingDomain editingDomain = modelState.getEditingDomain();
        Optional<GPoint> relativeLocation = getRelativeLocation(operation);
        Page page = getContainerPage(operation);

        RefTransition newRefTransition = PtnetFactory.eINSTANCE.createRefTransition();
        initializeElement(newRefTransition);

        NodeGraphics shape = createShape(idGenerator.getOrCreateId(newRefTransition),
            relativeLocation, GraphUtil.dimension(25, 60));
        newRefTransition.setNodegraphics(shape);

        setDiNeROSRefTransitionInfo(newRefTransition, page.getId());

        Command refTranCommand = AddCommand.create(editingDomain, page,
            PtnetPackage.Literals.PAGE__REFERENCE_TRANSITION, newRefTransition);

        return Optional.of(refTranCommand);
    }

    protected void setDiNeROSRefTransitionInfo(RefTransition refTransition, String pageNodeId){
        ToolInfoNode toolInfo = PtnetFactory.eINSTANCE.createToolInfoNode();
        setDiNeROSToolSpecifics(toolInfo, pageNodeId);
        refTransition.getToolspecific().add(toolInfo);
    }

}
