package org.eclipse.glsp.example.javaemf.handler;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GDimension;
import org.eclipse.glsp.graph.GPoint;
import org.eclipse.glsp.server.emf.EMFOperationHandler;
import org.eclipse.glsp.server.operations.ChangeBoundsOperation;
import org.eclipse.glsp.server.types.ElementAndBounds;
import org.pnml.emf.ptnet.*;

import java.util.Optional;

public class PnmlChangeBoundsOperationHandler extends EMFOperationHandler<ChangeBoundsOperation> {

    @Override
    public Optional<Command> createCommand(ChangeBoundsOperation operation) {
        EditingDomain editingDomain = modelState.getEditingDomain();
        CompoundCommand compoundCommand = new CompoundCommand();

        for (ElementAndBounds element : operation.getNewBounds()) {
            NodeGraphics shape = createShape(element.getElementId(), Optional.of(element.getNewPosition()) , element.getNewSize());
            EObject modelElement = modelState.getIndex().getEObject(element.getElementId()).orElseThrow();

            EObject container = modelElement.eContainer();

            if(!outOfBounds(element, container)){
                Command nodeGraphicsCommand = SetCommand.create(editingDomain, modelElement,
                    PtnetPackage.Literals.POSITIONABLE_OBJECT__NODEGRAPHICS, shape);

                compoundCommand.append(nodeGraphicsCommand);
            }
        }

        return Optional.of(compoundCommand);
    }

    protected NodeGraphics createShape(final String elementId, final Optional<GPoint> relativeLocation, final GDimension size) {
        Position position = PtnetFactory.eINSTANCE.createPosition();
        position.setX(relativeLocation.map(GPoint::getX).orElse(0.0).intValue());
        position.setY(relativeLocation.map(GPoint::getY).orElse(0.0).intValue());

        Dimension dimension = PtnetFactory.eINSTANCE.createDimension();
        dimension.setX((int) size.getWidth());
        dimension.setY((int) size.getHeight());

        NodeGraphics newShape = PtnetFactory.eINSTANCE.createNodeGraphics();
        newShape.setPosition(position);
        newShape.setDimension(dimension);

        return newShape;
    }


    protected boolean outOfBounds(ElementAndBounds element, EObject container) {

        // top level pages can always be repositioned
        if(PtnetPackage.eINSTANCE.getPetriNet().isSuperTypeOf(container.eClass())){
            return false;
        }

        Page containerPage = (Page) container;

        double pageWidth = Optional.ofNullable(containerPage.getNodegraphics()).map(NodeGraphics::getDimension).map(Coordinate::getX).orElse(PNMLModelTypes.DefaultSizes.PageWidth);
        double pageHeight = Optional.ofNullable(containerPage.getNodegraphics()).map(NodeGraphics::getDimension).map(Coordinate::getY).orElse(PNMLModelTypes.DefaultSizes.PageHeight);


        if (element.getNewPosition().getX() < 0
            || element.getNewPosition().getX() + element.getNewSize().getWidth() > pageWidth
            || element.getNewPosition().getY() < 0
            || element.getNewPosition().getY() + element.getNewSize().getHeight() > pageHeight) {
            return true;
        }
        return false;
    }
}
